import { world, system, Player, Scoreboard, ItemStack } from '@minecraft/server';
import { ActionFormData, ModalFormData, MessageFormData } from "@minecraft/server-ui";
import config from "./configs.js";

if (!(world.scoreboard.getObjective(config.BountyScoreboard) && world.scoreboard.getObjective(config.CurrencyScoreboard))) {
    world.scoreboard.addObjective(config.BountyScoreboard, config.BountyScoreboard);
    world.scoreboard.addObjective(config.CurrencyScoreboard, config.CurrencyScoreboard);
}

function getScore(target, objective) {
    try {
        target.runCommand(`scoreboard players add @s ${objective} 0`);
        return world.scoreboard.getObjective(objective).getScore(target);
    } catch {
        return 0;
    }
}

function removeScore(amount, target, objective) {
    if (!(target instanceof Player)) return;
    target.runCommand(`scoreboard players remove @s ${objective} ${amount}`);
}

function addScore(amount, target, objective) {
    if (!(target instanceof Player)) return;
    target.runCommand(`scoreboard players add @s ${objective} ${amount}`);
}

async function search(viewer) {
    const onlinePlayers = world.getPlayers().map(p => p.nameTag);
    const playerBalance = getScore(viewer, config.CurrencyScoreboard);

    if (onlinePlayers.length === 0) {
        viewer.sendMessage("[§cBounty System§r] > §4No players available to set a bounty on.");
        return;
    }

    const dropdownOptions = ["§7[§cChoose a Target§7]", ...onlinePlayers];

    const form = new ModalFormData()
        .title("§l§dSet Bounty")
        .dropdown("§aChoose a Player", dropdownOptions)
        .textField(
            `§7Enter Bounty Amount (Your Balance: §6${playerBalance} ${config.CurrencyScoreboard}§7):`,
            "Bounty Amount"
        )
        .submitButton("§aConfirm Bounty");

    form.show(viewer).then(({ formValues, canceled }) => {
        if (canceled) return;

        const selectedIndex = formValues[0];
        const BountyAmount = formValues[1];

        if (isNaN(BountyAmount) || BountyAmount <= 0) {
            denyUI(viewer, search, `\n§c§lInvalid bounty amount`);
            return;
        }

        if (BountyAmount > playerBalance) {
            denyUI(viewer, search, `\n§c§lNot enough ${config.CurrencyScoreboard}`);
            return;
        }

        if (selectedIndex === 0) {
            openPlayerSelectionMenu(viewer, BountyAmount);
        } else {
            const selectedPlayerName = dropdownOptions[selectedIndex];
            const selectedPlayer = world.getPlayers().find(p => p.nameTag === selectedPlayerName);

            if (selectedPlayer) {
                removeScore(BountyAmount, viewer, config.CurrencyScoreboard);
                addScore(BountyAmount, selectedPlayer, config.BountyScoreboard);
                world.sendMessage(
                    `[§cBounty System§r] > §4${selectedPlayer.nameTag}'s Bounty Increased By §6$${BountyAmount} ${config.CurrencyScoreboard}!`
                );
            } else {
                viewer.sendMessage("[§cBounty System§r] > §4An error occurred, player not found.");
            }
        }
    });
}

function openPlayerSelectionMenu(viewer, bountyAmount) {
    const onlinePlayers = world.getPlayers();
    if (onlinePlayers.length === 0) {
        viewer.sendMessage("[§cBounty System§r] > §4No players available to set a bounty on.");
        return;
    }

    const form = new ActionFormData()
        .title("§l§dSelect a Target")
        .body(`§7Choose a player to set a bounty of §6${bountyAmount} ${config.CurrencyScoreboard}§7 on:`);

    onlinePlayers.forEach(player => {
        form.button(player.nameTag);
    });

    form.show(viewer).then(({ selection, canceled }) => {
        if (canceled || selection === undefined) return;

        if (selection === onlinePlayers.length) {
            viewer.sendMessage("[§cBounty System§r] > §4Canceled bounty set.");
            return;
        }

        const selectedPlayer = onlinePlayers[selection];

        if (selectedPlayer) {
            removeScore(bountyAmount, viewer, config.CurrencyScoreboard);
            addScore(bountyAmount, selectedPlayer, config.BountyScoreboard);
            world.sendMessage(
                `[§cBounty System§r] > §4${selectedPlayer.nameTag}'s Bounty Increased By §6$${bountyAmount} ${config.CurrencyScoreboard}!`
            );
        } else {
            viewer.sendMessage("[§cBounty System§r] > §4An error occurred, player not found.");
        }
    });
}

async function denyUI(target, nextUI, uiData) {
    target.playSound('error');
    nextUI(target, uiData);
}

world.afterEvents.entityDie.subscribe((event) => {
    const victim = event.deadEntity;
    const killer = event.damageSource?.damagingEntity;

    // Ensure both victim and killer are players and they are not the same entity
    if (!(victim instanceof Player) || !(killer instanceof Player) || victim === killer) return;

    const bounty = getScore(victim, config.BountyScoreboard);
    if (bounty <= 0) return;

    const skullItem = new ItemStack(
        victim.name === "N0TAssassin" ? "minecraft:skeleton_skull" : "minecraft:player_head",
        1
    );

    skullItem.nameTag = `§c${victim.name}'s Skull`;
    skullItem.setLore([
        `§7Victim: §f${victim.name}`,
        `§7Killed by: §f${killer.name}`,
        `§7Bounty: §6${bounty} ${config.CurrencyScoreboard}`,
    ]);

    victim.dimension.spawnItem(skullItem, victim.location);
});

function claimBounty(source, itemStack) {
    if (!(itemStack.typeId === "minecraft:skeleton_skull" || itemStack.typeId === "minecraft:player_head")) return false;

    const lore = itemStack.getLore();
    if (!lore || !lore.some(line => line.includes('§7Bounty:'))) return false;

    const victimName = lore.find(line => line.startsWith('§7Victim:'))?.split('§f')[1];
    const bountyAmount = parseInt(lore.find(line => line.includes('§7Bounty:'))?.split('§6')[1] || 0);
    const killerName = lore.find(line => line.startsWith('§7Killed by:'))?.split('§f')[1];

    if (!victimName || isNaN(bountyAmount) || bountyAmount <= 0) {
        source.sendMessage("[§cBounty System§r] > §4Invalid bounty skull.");
        source.runCommand('playsound note.bass @s');
        return false;
    }

    if (victimName === source.name) {
        source.sendMessage("[§cBounty System§r] > §4You cannot claim a bounty on yourself.");
        source.runCommand('playsound note.bass @s');
        return false;
    }

    const victim = world.getAllPlayers().find(player => player.name === victimName);

    if (!victim) {
        source.sendMessage(`[§cBounty System§r] > §4Victim ${victimName} is offline.`);
        source.runCommand('playsound note.bass @s');
        return false;
    }

    const victimBountyScore = getScore(victim, config.BountyScoreboard);

    if (victimBountyScore <= 0) {
        source.sendMessage(`[§cBounty System§r] > §4${victimName} no longer has a bounty.`);
        source.runCommand('playsound note.bass @s');
        return false;
    }

    let remainder;
    if (victimBountyScore < bountyAmount) {
        remainder = victimBountyScore;
        addScore(victimBountyScore, source, config.CurrencyScoreboard);
        removeScore(victimBountyScore, victim, config.BountyScoreboard);
        world.sendMessage(
            `[§cBounty System§r] > §4${source.name} claimed the bounty on §e${victimName}§4 for §6${victimBountyScore} ${config.CurrencyScoreboard}!`
        );
    } else {
        remainder = bountyAmount;
        addScore(bountyAmount, source, config.CurrencyScoreboard);
        removeScore(bountyAmount, victim, config.BountyScoreboard);
        world.sendMessage(
            `[§cBounty System§r] > §4${source.name} claimed the bounty on §e${victimName}§4 for §6${bountyAmount} ${config.CurrencyScoreboard}!`
        );
    }

    updateLore(source, itemStack, victimName, killerName || source.name, remainder);

    source.runCommand('playsound random.orb @s');
    source.sendMessage("[§cBounty System§r] > §4Bounty claimed successfully!");
    return true;
}

function updateLore(source, itemStack, victimName, killerName, remainder) {
    let updatedLore = [
        `§7Victim: §f${victimName}`,
        `§7Killed by: §f${killerName}`,
    ];
    if (remainder <= 0) updatedLore.push("§7Bounty: §aClaimed");

    const inventory = source.getComponent("minecraft:inventory").container;

    for (let i = 0; i < inventory.size; i++) {
        const slotItem = inventory.getItem(i);
        if (slotItem && slotItem.typeId === itemStack.typeId && slotItem.getLore().join() === itemStack.getLore().join()) {
            slotItem.setLore(updatedLore);
            inventory.setItem(i, slotItem);
            break;
        }
    }
}

world.afterEvents.itemUse.subscribe(({ source, itemStack }) => {
    if (
        !(itemStack.typeId === "minecraft:skeleton_skull" || itemStack.typeId === "minecraft:player_head") ||
        !itemStack.getLore()?.some(line => line.includes("§7Bounty:"))
    ) {
        return;
    }

    const ui = new MessageFormData();
    ui.title("§aClaim Bounty");
    ui.body("§7Do you want to claim the bounty on this skull?");
    ui.button1("§4No");
    ui.button2("§aYes");

    ui.show(source).then(({ selection }) => {
        if (selection === 1) {
            const claimed = claimBounty(source, itemStack);
        }
    });
});

world.afterEvents.itemUse.subscribe(({ source, itemStack }) => {
    if (itemStack.typeId == 'bounty:bounty_contract') openMainBountyMenu(source);
});

system.afterEvents.scriptEventReceive.subscribe(({ id, sourceEntity }) => {
    if (id === "bounty:open") openMainBountyMenu(sourceEntity);
});

function openMainBountyMenu(player) {
    const ui = new MessageFormData();
    ui.title("§cBounty Menu");
    ui.body("What would you like to do?");
    ui.button1("§aView Top 10 Bounties");
    ui.button2("§aSet a Bounty");

    ui.show(player).then(({ selection }) => {
        if (selection === 0) {
            showTop10Bounties(player);
        } else if (selection === 1) {
            search(player);
        }
    });
}

world.beforeEvents.itemUseOn.subscribe((event) => {
    const { itemStack, source, block } = event;

    if (
        itemStack &&
        (itemStack.typeId === "minecraft:skeleton_skull" || itemStack.typeId === "minecraft:player_head") &&
        itemStack.getLore()?.some((line) => line.includes("§7Victim"))
    ) {
        if (source.isSneaking) {
            event.cancel = true;
            return;
        }

        const blockType = block?.typeId;

        if (
            blockType === "minecraft:frame" ||
            blockType === "minecraft:glow_frame" ||
            blockType === "minecraft:chest" ||
            blockType === "minecraft:trapped_chest" ||
            blockType === "minecraft:barrel" ||
            blockType === "minecraft:hopper" ||
            blockType === "minecraft:furnace" ||
            blockType === "minecraft:smoker" ||
            blockType === "minecraft:crafting_table" ||
            blockType === "minecraft:cartography_table" ||
            blockType === "minecraft:anvil" ||
            blockType === "minecraft:crafter" ||
            blockType === "minecraft:lever" ||
            blockType === "minecraft:smithing_table" ||
            blockType === "minecraft:chipped_anvil" ||
            blockType === "minecraft:damaged_anvil" ||
            blockType.includes("table") ||
            blockType.includes("button") ||
            blockType.includes("chest") ||
            blockType.includes("shulker_box") ||
            blockType.includes("stand") ||
            blockType.includes("sign") ||
            blockType.includes("furnace") ||
            blockType.includes("door") ||
            blockType === "minecraft:lectern" ||
            blockType === "minecraft:grindstone"
        ) {
            return;
        }

        event.cancel = true;
    }
});

function showTop10Bounties(player) {
    const bountyObjective = world.scoreboard.getObjective(config.BountyScoreboard);
    if (!bountyObjective) {
        player.sendMessage("§cBounty scoreboard not found.");
        return;
    }

    const onlinePlayers = world.getAllPlayers();
    const scores = [];

    for (const onlinePlayer of onlinePlayers) {
        const score = bountyObjective.getScore(onlinePlayer);
        if (score > 0) {
            scores.push({ name: onlinePlayer.name, score });
        }
    }

    scores.sort((a, b) => b.score - a.score);
    const top10 = scores.slice(0, 10);

    const ui = new ActionFormData();
    ui.title("§gTop 10 Bounties");

    if (top10.length === 0) {
        ui.body("No bounties available.");
    } else {
        ui.body(
            top10
                .map((entry, index) => `§a${index + 1}. §f${entry.name}: §6${entry.score} ${config.CurrencyScoreboard}`)
                .join("\n")
        );
    }
    ui.button("§4Close");

    ui.show(player);
}
